package com.yakindu.bridges.ea.example.codegen;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.VisibilityKind;

import com.google.common.base.Joiner;

/**
 * A helper class for code generation templates.
 */
public class CodegenUtils {

	public static String getPackageName(PackageableElement e) {
		return e.getNearestPackage().getQualifiedName().replace("::", ".");
	}

	public static String getVisibilityAsString(NamedElement e) {
		final VisibilityKind visibility = e.getVisibility();
		if (visibility == null)
			return VisibilityKind.PACKAGE_LITERAL.getName().toLowerCase(); // package as default visibility
		return visibility.getName().toLowerCase();
	}

	public static String getImplementedInterfacesAsString(Class c) {
		final String interfaceNamesAsList = getNamesAsList(c.getImplementedInterfaces());
		return interfaceNamesAsList.isEmpty() ? "" : " implements " + interfaceNamesAsList;
	}

	public static String getSuperClassAsString(Class c) {
		final String superClassNamesAsList = getNamesAsList(c.getSuperClasses());
		return superClassNamesAsList.isEmpty() ? "" : " extends " + superClassNamesAsList;
	}

	private static String getNamesAsList(Iterable<? extends NamedElement> elements) {
		final List<String> list = new ArrayList<>();
		for (NamedElement element : elements)
			list.add(element.getName());
		return Joiner.on(", ").join(list);
	}
}
