package com.yakindu.ea.examples.plugin.generators;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.UMLPackage;

public class ResourceInfo {

	public final Resource resource;

	private int maxDepth = 0;
	private int packages = 0;
	private int packagedElements = 0;

	public ResourceInfo(Resource resource) {
		this.resource = resource;
	}

	public String getElementName(PackageableElement element, int depth) {
		maxDepth = Math.max(maxDepth, depth);
		visitType(element.eClass());
		return element.getName();
	}

	private void visitType(EClass eClass) {
		if (UMLPackage.Literals.PACKAGE.isSuperTypeOf(eClass)) {
			packages++;
		} else if (UMLPackage.Literals.PACKAGEABLE_ELEMENT.isSuperTypeOf(eClass)) {
			packagedElements++;
		}
	}

	public List<? extends PackageableElement> getChildren(PackageableElement element) {
		if (element instanceof Package) {
			return ((Package) element).getPackagedElements().stream().filter(c -> nameIsSet(c))
					.collect(Collectors.toList());
		} else if (element instanceof Component) {
			return ((Component) element).getPackagedElements().stream().filter(c -> nameIsSet(c))
					.collect(Collectors.toList());
		} else if (element instanceof Interface) {
			return ((Interface) element).getNestedClassifiers().stream().filter(c -> nameIsSet(c))
					.collect(Collectors.toList());
		} else if (element instanceof Class) {
			return ((Class) element).getNestedClassifiers().stream().filter(c -> nameIsSet(c))
					.collect(Collectors.toList());
		}
		return Collections.emptyList(); // only show packageable elements in the tree
	}

	private boolean nameIsSet(PackageableElement e) {
		final String name = e.getName();
		return name != null && !name.isEmpty();
	}

	public int getMaxDepth() {
		return maxDepth + 1;
	}

	public int getPackages() {
		return packages;
	}

	public int getPackagedElements() {
		return packagedElements;
	}
}
